/*
 * srrctl.c: module control facility
 *
 * $Id: srrctl.c 132 2011-07-01 14:35:01Z meaton $
 */

#include <errno.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

#include "srr.h"

void Usage(FILE* out)
{
	fprintf(out, "%s", "usage: srrctl [-h] [-t level] [-u] [-kpid,signo]\n");
}

void Help()
{
	Usage(stdout);

	fprintf(stdout, "%s",
		"\n"
		"options: \n"
		"  -h   this useful help message\n"
		"  -t   set trace output to 'level' (if compiled with\n"
		"          debugging enabled) - currently only trace levels\n"
		"          1 (on) and 0 (off) are defined\n"
		"  -u   unload  (sets the module usage count to zero, so\n"
		"          that the module can be unloaded if its crashed)\n"
		"  -k   kill, sends signo to pid, or all pids if pid is zero, if they\n"
		"          are registered with the module\n"
		"\n"
		"Utility to send control messages to the SRR module (/dev/srr).\n"
		);
}

int main(int argc, char* argv[])
{
	int opt, waittime = 0;

	while((opt = getopt(argc, argv, "ht:uk:w:")) != -1)
	{
		switch(opt)
		{
		case 'h':
			Help();
			exit(0);

		case 't':
			if(SrrDebug(atoi(optarg)) == -1) {
				fprintf(stderr, "SrrDebug() failed: [%d] %s\n",
					errno, strerror(errno));
				exit(1);
			}
			break;

		case 'u':
			if(SrrUnload() == -1) {
				fprintf(stderr, "SrrUnload() failed: [%d] %s\n",
					errno, strerror(errno));
				exit(1);
			}
			break;

		case 'k': {
			int pid;
			int signo;
			if(2 != sscanf(optarg, "%d,%d", &pid, &signo)) {
				fprintf(stderr, "invalid argument for -k: %s\n", optarg);
				exit(1);
			}
			if(SrrKill(pid, signo) == -1) {
				fprintf(stderr, "SrrKill() failed: [%d] %s\n",
					errno, strerror(errno));
				exit(1);
			}
			break;
		case 'w':
			waittime = atoi (optarg);
			break;
		}


		default:
			Usage(stderr);
			exit(1);
		}
	}

	if(argc == 1) {
		Usage(stderr);
		exit(1);
	}

	if (waittime)
		sleep (waittime);
	
	return 0;
}

