/*
 * tserver.c: test program
 *
 * Copyright (C) 2000 Cogent Real-Time Systems Inc.
 *
 * $Id: ttimers.c 132 2011-07-01 14:35:01Z meaton $
 */

#include <assert.h>
#include <errno.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <signal.h>
#include <sys/types.h>
#include <sys/wait.h>

#include <sys/kernel.h>

void SigAlarmHandler(int signo)
{
	signo = signo;
	printf("sig alarmed\n");
}

void print_its(const char* lead, struct itimerspec* its)
{
	printf("%svalue %ld,%ld interval %ld,%ld\n",
		lead,
		its->it_value.tv_sec, its->it_value.tv_nsec,
		its->it_interval.tv_sec, its->it_interval.tv_nsec
		);
}

void print_ts(const char* lead, struct timespec* ts)
{
	printf("%s %ld,%ld\n", lead, ts->tv_sec, ts->tv_nsec);
}

void TimerSet(time_t tid, int sec)
{
	struct itimerspec value;
	struct itimerspec ovalue;
	struct itimerspec nvalue;

	value.it_value.tv_sec = sec;
	value.it_value.tv_nsec = 0;
	value.it_interval.tv_sec = 0;
	value.it_interval.tv_nsec = 0;

	print_its("settime value: ", &value);

	if(timer_settime (tid, TIMER_ADDREL, &value, &ovalue) == -1)
	{
		printf ("timer_settime() failed: [%d] %s\n", errno, strerror(errno));
		exit (1);
	}

	print_its("old value: ", &ovalue);

	if(timer_gettime (tid, &nvalue) == -1)
	{
		printf ("timer_gettime() failed: [%d] %s\n", errno, strerror(errno));
		exit (1);
	}
	assert(nvalue.it_interval.tv_sec  ==  value.it_interval.tv_sec);
	assert(nvalue.it_interval.tv_nsec == value.it_interval.tv_nsec);
}

int main()
{
	struct timespec	before;
	struct timespec	after;
	timer_t	tid;

	signal(SIGALRM, SigAlarmHandler);

	if ((tid = timer_create (CLOCK_REALTIME, 0)) == -1)
	{
		printf ("timer_create() failed: [%d] %s\n", errno, strerror(errno));
		exit (1);
	}


	TimerSet(tid, 2);
	assert(sleep(1) == 0);

	TimerSet(tid, 0); // cancel

	assert(sleep(2) == 0); // was cancelled

	TimerSet(tid, 10);
	sleep(2);

	clock_gettime(CLOCK_REALTIME, &before);

	print_ts("before: ", &before);

	TimerSet(tid, 3);
	
	assert(Receive(0, 0, 0) == -1);
	assert(errno == EINTR);

	clock_gettime(CLOCK_REALTIME, &after);

	print_ts("after: ", &after);

	after.tv_sec  -= before.tv_sec;
	after.tv_nsec -= before.tv_nsec;

	print_ts("diff: ", &after);

	TimerSet(tid, 0);
	return 0;
}

