/*
 * This is the opcode list for the little interpreter used to perform
 * interrupt-time processing.  Never ever add things into the middle of
 * this table, always at the end before SBC_LAST.
 * If you modify this table, also modify SrrBC_OpInfo in the file
 * srr_bytecode.c
 */

enum SrrBC_Opcode
{
	SBC_MAPA = 1,		/* 1 */
	SBC_MAPS,
	SBC_MAPP,
	SBC_LDREG,			/* 4 */
	SBC_LDREGA,
	SBC_LDREGS,
	SBC_LDREGP,
	SBC_STREGA,			/* 8 */
	SBC_STREGS,
	SBC_STREGP,
	SBC_ADD,
	SBC_SUB,			/* 12 */
	SBC_MUL,
	SBC_DIV,
	SBC_NOT,
	SBC_AND,			/* 16 */
	SBC_OR,
	SBC_SHL,
	SBC_SHR,
	SBC_CPYR,			/* 20 */
	SBC_CPYAS,
	SBC_CPYSA,
	SBC_READ,
	SBC_WRITE,			/* 24 */
	SBC_INC,
	SBC_DEC,
	SBC_PROXY,
	SBC_ATTACH,			/* 28 */
	SBC_DETACH,
	SBC_DISABLE,
	SBC_ENABLE,
	SBC_XOR,			/* 32 */
	SBC_HANDLED,

	/* Last opcode. Not used except for error checking */
	SBC_LAST
};

typedef struct SrrBC_Address
{
	uint32_t	offset;
	uint8_t		*buf;
	uint32_t	size;
} SrrBC_Address;

typedef struct SrrBC_Storage
{
	uint8_t		*buf;
	uint32_t	size;
} SrrBC_Storage;

#define SBC_NREG	4
#define SBC_NADDR	8
#define SBC_NSTOR	8
#define SBC_NPORT	4
#define SBC_NRES	(SBC_NADDR + SBC_NPORT)

typedef struct SrrBC_Program
{
	uint32_t		reg[SBC_NREG];
	SrrBC_Address	addr[SBC_NADDR];
	SrrBC_Storage	stor[SBC_NSTOR];
	struct resource	*resource[SBC_NRES];

	uint8_t			**code;
	uint32_t		*codelen;
	uint32_t		ncode;
	uint32_t		maxcode;
} SrrBC_Program;

#define VALIDREG(_n)		((_n) < SBC_NREG && (_n) >= 0)
#define VALIDADNO(_n)		((_n) < SBC_NADDR && (_n) >= 0)
#define VALIDSTNO(_n)		((_n) < SBC_NSTOR && (_n) >= 0)
#define VALIDADDR(_i,_n)	(VALIDADNO(_n) && (_i)->program.addr[_n].buf)
#define VALIDSTOR(_i,_n)	(VALIDSTNO(_n) && (_i)->program.stor[_n].buf)

#define SBPUT8(_value,_buf,_off) (_buf[_off] = (int8_t)(_value), sizeof(uint8_t))
#define SBPUT16(_value,_buf,_off) (*((uint16_t*)(&_buf[_off])) = (uint16_t)(_value), sizeof(uint16_t))
#define SBPUT32(_value,_buf,_off) (*((uint32_t*)(&_buf[_off])) = (uint32_t)(_value), sizeof(uint32_t))

#define SBGET8(_var,_buf,_off) ((_var) = (_buf)[_off], sizeof(uint8_t))
#define SBGET16(_var,_buf,_off) ((_var) = *((uint16_t*)&(_buf)[_off]), sizeof(uint16_t))
#define SBGET32(_var,_buf,_off) ((_var) = *((uint32_t*)&(_buf)[_off]), sizeof(uint32_t))

extern int Srr_IRQUsed[16];
