/*
 * srr_time.h: implementation of POSIX timers w/ QNX extensions
 *
 *  Linux Loadable Module Implementation of QNX Send/Receive/Reply
 *	message passing, and related functions.
 *
 *	QNX is a trademark of QSSL, Ltd. <www.qnx.com>
 *
 * Copyright (C) 2000 Cogent Real-Time Systems Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software in a file called 'COPYING'; if not, write to:
 * 	Free Software Foundation, Inc.,
 * 	59 Temple Place, Suite 330,
 * 	Boston, MA 
 * 	02111-1307  USA
 *
 * $Id: srr_time.h 3613 2009-02-01 19:58:01Z andrew $
 */

#include "sysdep.h"
#include "srr_taskLocks.h"

#ifndef SRR_TIME_H
#define SRR_TIME_H

#define NR_TIMERS	64 /* this should be configurable on load time */

typedef struct SrrPt /* POSIX timer */
{
	pid_t			owner;
	clock_t			clock;
	int			flags;
	struct sigevent		event;
	struct itimerspec 	timespec;
	struct timer_list 	timer;
	unsigned long		interval; /* = timespec_to_jiffies(&timeval.it_interval) */
	unsigned long  		usec;
#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,0)
	struct tq_struct   qtask;
#else
	struct work_struct qtask;
#endif
	atomic_t	qcount;
	atomic_t	is_used;
	atomic_t	is_scheduled;
	atomic_t	is_allocated;
	int		arm_count;
	int		fire_count;
	int		trigger_count;
	int		queued;
	int		queue_fail;
	int		bottom_halfs;
	unsigned long int	expires;		/* expiry time, in jiffies */
	int		tid;
} SrrPt;

int		SrrPt_DeleteAll(pid_t pid);
int		SrrPt_Delete(pid_t pid, srr_timer_t tid);
srr_timer_t SrrPt_Create(pid_t owner, struct sigevent* se);

int SrrModTimerCreate	(struct SrrTm* map, struct SrrTask* task);
int SrrModTimerDelete	(struct SrrTm* map, struct SrrTask* task);
int SrrModTimerSet 	(struct SrrTm* map, struct SrrTask* task);
int SrrModTimerGet 	(struct SrrTm* map, struct SrrTask* task);
int SrrModClockGetres	(struct SrrTm* map, struct SrrTask* task);


#ifdef WORK_DATA_IS_VOID
void SrrPt_BottomHalf (void* data);
#else
void SrrPt_BottomHalf (struct work_struct *work);
#endif // WORK_DATA_IS_VOID
#endif

