/*
 * srr_util.h:
 *
 *  Linux Loadable Module Implementation of QNX Send/Receive/Reply
 *	message passing, and related function.
 *
 *	QNX is a trademark of QSSL, Ltd. <www.qnx.com>
 *
 * Copyright (C) 2000 Cogent Real-Time Systems Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software in a file called 'COPYING'; if not, write to:
 * 	Free Software Foundation, Inc.,
 * 	59 Temple Place, Suite 330,
 * 	Boston, MA 
 * 	02111-1307  USA
 *
 * $Id: srr_util.h 3612 2009-01-30 16:55:39Z andrew $
 */

#ifndef SRR_UTIL_H
#define SRR_UTIL_H

#include "sysdep.h"
#include "srr_taskLocks.h"

#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,2,0)
#	include <asm/uaccess.h>
#endif


/*
 * Internal oops, won't deliberately halt the system but something is
 * seriously wrong, probably an implementation error. An Oops() should
 * never happen if code is correct, as distinguished from a printk(WARN...)
 * which mark exceptional conditions in a correctly running system
 * (such as out-of-memory errors).
 */

#define Oops(fmt, args...) \
   printk(KERN_CRIT "srr(%d): oops - " fmt,  current->pid, ##args);

#define CHKMAP(call) if(!map) { Oops(call " called with no map!\n"); return 0; }

#define Static
#define Extern

#ifdef LOCK_SAFE_COPY
extern struct mutex copyLock;
#endif

/*
 * Macros to help debugging
 */
#undef PDEBUG	          /* undef it, just in case */
#ifdef SRR_DEBUG
#	ifdef __KERNEL__
		/* This one if debugging is on, and kernel space */
		int srr_debug_trace(int level);
		extern int srr_debug_level;
#		define PDEBUG(fmt, args...) \
			if(srr_debug_level > 1) \
				printk( KERN_ERR "%ld: srr(%d): " fmt, jiffies, current->pid, ##args)
#		define KDEBUG(fmt, args...) \
			if(srr_debug_level > 0) \
				printk( KERN_ERR "%ld: srr(%d): " fmt, jiffies, current->pid, ##args)
#	else
		/* This one for user space */
#		define PDEBUG(fmt, args...) fprintf(stderr, fmt , ## args)
#		define KDEBUG(fmt, args...) fprintf(stderr, fmt , ## args)
#	endif
#else
#	define srr_debug_trace(X) 0
#	define PDEBUG(fmt , args...) /* not debugging: nothing */
#  define KDEBUG(fmt , args...) /* <AS>: not debugging: nothing */
#endif

#undef NDEBUG
#define NDEBUG(fmt, args...) /* nothing: it's a placeholder */

#ifndef SRR_MAJOR
#	define SRR_MAJOR 10		/* misc minor device by default */
#endif

/*
 * memory xfer: wrappers for copying data between user and kernel space
 */

int cpy_u2k(void* dst, const void* src, size_t size);
int cpy_k2u(void *dst, const void* src, size_t size);

/*
 * memory xfer: get variable sized data from user space into a buffer
 */

char*	str_cpy_u2k(const char* src, size_t size);
void	str_cpy_release(void* buffer);

void hexdump(void *data, size_t length);

#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,0)
#define SPIN_FLAG_TYPE	unsigned long
#else
#define SPIN_FLAG_TYPE	int
#endif

#endif /* SRR_INTERNAL_H */

